# -*- coding: utf-8 -*-
import logging
from django.core.validators import RegexValidator
from django.utils.deconstruct import deconstructible
from django.core.exceptions import ValidationError


AlphaNumericValidator = RegexValidator(r'^[a-zA-Z0-9]*$',
                                       message='Este campo debe ser alfanumérico.',
                                       code='Inválido')

AlphaNumericValidatorWithSpaces = RegexValidator(r'^[A-Za-zÑñáéíóúÁÉÍÓÚ0-9 \-\–]+$',
                                                 message='Este campo solo permite números, letras, guiones y espacios.',
                                                 code='Inválido')

NumericValidatorWithSpaces = RegexValidator(r'^[0-9]*$',
                                            message='Este campo debe ser numérico.',
                                            code='Inválido')

AlphaValidatorWithSpaces = RegexValidator(r'^[a-zA-Z- ñÑ-áéíóúÁÉÍÓÚ]+$',
                                          message='Este campo sólo permite letras.',
                                          code='Inválido')


@deconstructible
class FileSizeValidator:
    code = 'invalid'

    def __init__(self, max_size=None):
        if max_size:
            self.max_size = max_size
        else:
            raise AttributeError('Debe ingresar el máximo peso del archivo')

    def __call__(self, value):
        logging.info('Calling {}'.format(value))

        size = value.file.size

        if size > self.max_size*1024*1024:
            raise ValidationError("Tamaño máximo permitido es %sMB" % str(self.max_size))

    def __eq__(self, other):
        return (
                isinstance(other, FileSizeValidator) and
                (self.max_size == other.max_size) and
                (self.accept == other.accept)
        )



def get_verbose_name_headers(fields, model) -> list:
    """
    :param fields: list
    :param model: django.db.models.Model
    :return: list
    """
    headers = []
    for field in fields:
        # noinspection Unresolved,PyProtectedMember
        model_fields = model._meta.get_fields()
        header = next((x.verbose_name.__str__().capitalize() for x in model_fields if x.name == field.column_name),
                      field.column_name)
        headers.append(header)

    return headers



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import logging
from django.utils import timezone


def delete_selected(instance, request, queryset):
    """
    :param instance: django.contrib.admin.ModelAdmin
    :param request: urllib.request.Request
    :param queryset: django.db.models.query.Queryset
    :return:
    """
    logging.info("delete_selected {}, {}".format(instance.__str__(), request.user.__str__()))
    queryset.update(deleted=True, deleted_at=timezone.now())


delete_selected.__name__ = 'Eliminar seleccionados'
delete_selected.short_description = 'Eliminar seleccionados'


def activate_selected(instance, request, queryset):
    """
    :param instance: django.contrib.admin.ModelAdmin
    :param request: urllib.request.Request
    :param queryset: django.db.models.query.Queryset
    :return:
    """
    logging.info("activate_selected {}, {}".format(instance.__str__(), request.user.__str__()))
    queryset.update(active=True)


activate_selected.__name__ = 'Activar seleccionados'
activate_selected.short_description = 'Activar seleccionados'


def inactivate_selected(instance, request, queryset):
    """
    :param instance: django.contrib.admin.ModelAdmin
    :param request: urllib.request.Request
    :param queryset: django.db.models.query.Queryset
    :return:
    """
    logging.info("inactivate_selected {}, {}".format(instance.__str__(), request.user.__str__()))
    queryset.update(active=False)


inactivate_selected.__name__ = 'Inactivar seleccionados'
inactivate_selected.short_description = 'Inactivar seleccionados'



from rest_framework import serializers
from rest_framework.fields import empty


class BaseModelSerializer(serializers.ModelSerializer):
    created_by = serializers.StringRelatedField(read_only=True)
    
    created_at = serializers.DateTimeField(read_only=True)

    updated_at = serializers.DateTimeField(read_only=True)
    
    def create(self, validated_data):
        request = self.context.get('request')

        if request:
            validated_data['created_by'] = request.user

        return super().create(validated_data)

    def __init__(self, instance=None, data=empty, **kwargs):
        request = kwargs.get('request')

        print('request is ', request)

        remove_fields = kwargs.pop('remove_fields', [])

        super().__init__(instance, data, **kwargs)

        if not isinstance(remove_fields, list):
            remove_fields = [remove_fields]

        for field_name in remove_fields:
            self.fields.pop(field_name)

    class Meta:
        exclude = ('deleted', 'deleted_at')



from django.utils.timezone import localtime
from import_export.widgets import DateTimeWidget
from django.conf import settings


class TzDateTimeWidget(DateTimeWidget):

    def render(self, value, obj=None):
        if settings.USE_TZ:
            value = localtime(value)
        return super(TzDateTimeWidget, self).render(value, obj)



from abc import abstractmethod
from django.db.models import QuerySet
from django.http import HttpResponse
from rest_framework.decorators import action
from rest_framework.response import Response
from fwbasemodelmodels import PERMISSION_EXPORT_CODE
import logging


class ExportModelMixinViewSet:
    resource = None

    def __init__(self, resource=None, **kwargs):
        """
        :param resource: import_export.resources.ModelResource
        """
        super().__init__(**kwargs)
        self.request = None
        self.resource = resource

    @action(detail=False, methods=['GET'])
    def export(self, request):

        queryset = self.get_queryset()

        name = queryset.model.__name__

        code = PERMISSION_EXPORT_CODE.format(str(name).lower())

        if not request.user.has_perm('app.{}'.format(code)):
            message = {
                'status': 'forbidden',
                'message': '{} no cuenta con permisos para exportar'.format(request.user)
            }
            return Response(message, status=403)

        try:
            if request.query_params:
                size = 1000

                params = self.request.query_params

                filters = self.get_queryset_filters(params)

                queryset = queryset.filter(**filters)
                queryset = queryset[:size][::-1]
            dataset = self.resource.export(queryset)
        except Exception as e:
            logging.exception(e)
            return HttpResponse({'error': 'Datos inváidos', 'exception': e.__str__()}, content_type='application/json')

        response = HttpResponse(dataset.xls, content_type='application/vnd.ms-excel')

        response['Content-Disposition'] = 'attachment; filename="{}s.xls"'.format(name)
        return response

    @abstractmethod
    def get_queryset(self) -> QuerySet:
        """
        return QuerySet
        """
        pass

    @staticmethod
    def get_queryset_filters(query_params):
        params = {}

        for value in query_params.keys():
            if query_params[value]:
                params[value] = query_params.get(value)

        return params


class CreatedByMixin:
    request = None

    def perform_create(self, serializer):
        serializer.save(created_by=self.request.user)



from django.contrib import admin


class BaseModelAdmin(admin.ModelAdmin):

    list_filter = ('active',)

    date_hierarchy = 'created_at'

    readonly_fields = ('created_by', 'created_at', 'updated_at')

    exclude = ()

    def get_list_display(self, request):
        list_display = super(BaseModelAdmin, self).get_list_display(request)

        if request.user.is_superuser:
            list_display += self.readonly_fields

        return list_display

    def get_exclude(self, request, obj=None):
        exclude = super(BaseModelAdmin, self).get_exclude(request)

        if not request.user.is_superuser:
            exclude += ('created_by', 'deleted', 'deleted_at')

        return exclude

    def get_list_filter(self, request):
        list_filter = super(BaseModelAdmin, self).get_list_filter(request)

        if request.user.is_superuser:
            list_filter += ('deleted',)

        return list_filter

    def get_queryset(self, request):
        if request.user.is_superuser:
            qs = self.model.admin_objects
        else:
            qs = self.model.objects

        ordering = self.get_ordering(request)

        if ordering:
            qs = qs.order_by(*ordering)

        return qs

    def get_search_results(self, request, queryset, search_term):
        """
        :param request: http.HttpRequest
        :param queryset: BaseQueryset
        :param search_term: str
        :return: Tuple[Any, bool]
        """
        if 'autocomplete' in request.path_info:
            queryset = queryset.filter(active=True)
        return super(BaseModelAdmin, self).get_search_results(request, queryset, search_term)


class BaseInlineMixin(object):
    exclude = BaseModelAdmin.exclude

    extra = 0



from django.conf import settings
from django.utils import timezone
from django.db.models.query import QuerySet


class BaseQuerySet(QuerySet):

    def delete(self):
        if settings.DEBUG:
            return super().delete()
        return self.update(deleted=True, deleted_at=timezone.now())



from django.contrib.auth import get_user_model
from import_export import resources, fields, widgets
from .widgets import TzDateTimeWidget
from .defs import get_verbose_name_headers

User = get_user_model()


class BaseResource(resources.ModelResource):
    def __init__(self, model=None, display_fields=None, exclude=None, append_exclude=()):
        super().__init__()

        if model:
            self.Meta.model = model

        if display_fields:
            self.Meta.fields = display_fields

        if hasattr(self.Meta, 'exclude'):
            if exclude:
                self.Meta.exclude = exclude

            self.Meta.exclude += append_exclude

    created_by = fields.Field(
        attribute="created_by",
        column_name="Creado por",
        readonly=True,
        widget=widgets.ForeignKeyWidget(User, "username")
    )

    created_at = fields.Field(column_name="Creado el", readonly=True,
                              widget=TzDateTimeWidget(), attribute='created_at')

    updated_at = fields.Field(column_name="Actualizado el", readonly=True,
                              widget=TzDateTimeWidget(), attribute='updated_at')

    def dehydrate_active(self, model) -> str:
        """
        :param model: django.db.models.Model
        :return: str
        """
        return 'si' if model.active else 'no'
    dehydrate_active.DEFAULT_RESOURCE_FIELD = 'active'

    def dehydrate_deleted(self, model) -> str:
        """
        :param model: django.db.models.Model
        :return: str
        """
        return 'si' if model.deleted else 'no'
    dehydrate_deleted.DEFAULT_RESOURCE_FIELD = 'deleted'

    def get_export_headers(self):
        return get_verbose_name_headers(self.get_fields(), self.Meta.model)

    class Meta:
        exclude = ('deleted', 'deleted_at')

        model = None
